
#include <ESP8266WiFi.h>
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"
#include <RBDdimmer.h>
/************************* WiFi Access Point *********************************/
#define WLAN_SSID       "GalaxyA8"
#define WLAN_PASS       "12345678"
/************************* ESP8266 pin config ********************************/
#define LED_BUILTIN 2 //D4, internal led
#define outputPin  12 //D6 PWM
#define zerocross  13 // D7 ZC
/************************* Adafruit.io Setup *********************************/
#define AIO_SERVER      "io.adafruit.com"
#define AIO_SERVERPORT  1883                   // use 8883 for SSL
#define AIO_USERNAME    "VoCom"
#define AIO_KEY         "7ec7f6fc12f546e9962a6d48f6576f4d"
/************ Global State (you don't need to change this!) ******************/
// Create an ESP8266 WiFiClient class to connect to the MQTT server.
WiFiClient client;   
// Setup the MQTT client class by passing in the WiFi client and MQTT server and login details.
Adafruit_MQTT_Client mqtt(&client, AIO_SERVER, AIO_SERVERPORT, AIO_USERNAME, AIO_KEY);
/****************************** Feeds ***************************************/
// Notice MQTT paths for AIO follow the form: <username>/feeds/<feedname>
// Setup feeds for subscribing to changes.
Adafruit_MQTT_Subscribe light_intensity = Adafruit_MQTT_Subscribe(&mqtt, AIO_USERNAME "/feeds/light");
/****************************** Dimmer Initialisation************************/
dimmerLamp dimmer(outputPin, zerocross); //initialase port for dimmer for ESP8266, ESP32, Arduino due boards
int outVal = 0; 
/*************************** Sketch Code ************************************/

void setup() {
  Serial.begin(115200);
  delay(10);

  Serial.println(F("Adafruit MQTT demo")); //cemu taj F?

  // Connect to WiFi access point.
  Serial.println(); Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WLAN_SSID);

  WiFi.begin(WLAN_SSID, WLAN_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.println("WiFi connected");
  Serial.println("IP address: "); 
  Serial.println(WiFi.localIP());

  // Setup MQTT subscription 
  mqtt.subscribe(&light_intensity);
  
  // Dimmer initialisation
  dimmer.begin(NORMAL_MODE, ON); //dimmer initialisation: name.begin(MODE, STATE) 
  Serial.println("Dimmer Program is starting...");
  Serial.println("Set value");
  
}

void loop() {
  // Ensure the connection to the MQTT server is alive (this will make the first
  // connection and automatically reconnect when disconnected). 
  
  MQTT_connect();

  // this is our 'wait for incoming subscription packets' busy subloop
  // try to spend your time here

  Adafruit_MQTT_Subscribe *subscription;
  while ((subscription = mqtt.readSubscription(5000))) {
    
    if (subscription == &light_intensity){
    Serial.print(F("Got: "));
    uint16_t intensity=atoi((char *)light_intensity.lastread);
    Serial.println(intensity);

    int preVal = outVal;
      if (intensity < 101 && intensity > -1) outVal = intensity;
      delay(200);
      dimmer.setPower(outVal); // setPower(0-100%);
      
        if (preVal != outVal)
        {
        Serial.print("lampValue -> ");
        Serial.print(dimmer.getPower());
        Serial.println("%");
        }
      delay(50);
    }   
 }
  
  // ping the server to keep the mqtt connection alive
  if(! mqtt.ping(3)) {
    mqtt.disconnect();
  }
  
}

// Function to connect and reconnect as necessary to the MQTT server.
// Should be called in the loop function and it will take care if connecting.
void MQTT_connect() {
  int8_t ret;

  // Stop if already connected.
  if (mqtt.connected()) {
    return;
  }

  Serial.print("Connecting to MQTT... ");

  uint8_t retries = 3;
  while ((ret = mqtt.connect()) != 0) { // connect will return 0 for connected
       Serial.println(mqtt.connectErrorString(ret));
       Serial.println("Retrying MQTT connection in 5 seconds...");
       mqtt.disconnect();
       delay(5000);  // wait 5 seconds
       retries--;
       if (retries == 0) {
         // basically die and wait for WDT to reset me
         while (1);
       }
  }
  Serial.println("MQTT Connected!");
}
